@@ -33,7 +33,7 @@ SingleCollectionTreeItemModel::SingleCollectionTreeItemModel( Collections::Colle
                                                               const QList<CategoryId::CatMenuId> &levelType )
     : m_collection( collection )
 {
-    m_rootItem = new CollectionTreeItem( m_collection, 0, this );
+    m_rootItem = new CollectionTreeItem( m_collection, nullptr, this );
     connect( collection, &Collections::Collection::updated, this, &SingleCollectionTreeItemModel::slotFilterWithoutAutoExpand ) ;
     m_collections.insert( m_collection->collectionId(), CollectionRoot( m_collection, m_rootItem ) );
     //we only have one collection that, by its very nature, is always expanded
